@@ -5,10 +5,11 @@ TODO:
 - add integration tests
 - expand to use other distance metrics
 """
+import dask.array as da
+import dask.dataframe as dd
 
 import csv
 import logging
-import math
 from collections import Counter
 
 import matplotlib.pylab as plt
@@ -31,22 +32,30 @@ class GAM:
     Args:
         k (int): number of clusters and centroids to form, default=2
         attributions_path (str): path for csv containing local attributions
-        attributions_df (pd.DataFrame, np.array, list): in-memory dataframe holding local attributions
-        feature_labels
+        attributions (pd.DataFrame, dd.DataFrame, np.array, da.array, list): in-memory dataframe holding local attributions
+        feature_labels (np.array, da.array, list): in-memory dataframe holding feature labels
         cluster_method: None, or callable, default=None
             None - use GAM library routines for k-medoids clustering
             callable - user provided external function to perform clustering
+        Using k-medoids from the library:
+            We provide slow & exact methods (e.g. PAM) and faster approximate methods
+            (e.g. Bandit-PAM) which can be specified by:
+        init_medoids: {'None', array} - how to pick initial medoids
+            None - use random selection
+            array - (features * k) representing initial medoids
+        swap_medoids: {'None', callable} - defaults to naive k-medoids
         distance: {‘spearman’, ‘kendall’}  distance metric used to compare attributions, default='spearman'
         use_normalized (boolean): whether to use normalized attributions in clustering, default='True'
         scoring_method (callable) function to calculate scalar representing goodness of fit for a given k, default=None
         max_iter (int): maximum number of iteration in k-medoids, default=100
         tol (float): tolerance denoting minimal acceptable amount of improvement, controls early stopping, default=1e-3
+        seed (int): seed for numpy random state, default=None
     """
 
     def __init__(
         self,
         k=2,
-        attributions_path="local_attributions.csv",
+        attributions_path=None,
         attributions=None,
         feature_labels=None,
         cluster_method=None,
@@ -55,6 +64,10 @@ class GAM:
         scoring_method=None,
         max_iter=100,
         tol=1e-3,
+        init_medoids=None,
+        swap_medoids=None,
+        verbose=False,
+        seed=None,
     ):
 
         self.attributions_path = attributions_path
@@ -78,9 +91,16 @@ class GAM:
                 distance
             )  # assume this is  metric listed in pairwise.PAIRWISE_DISTANCE_FUNCTIONS
 
+        self.scoring_method = scoring_method
+        self.init_medoids = init_medoids
+        self.swap_medoids = swap_medoids
+
         self.k = k
         self.max_iter = max_iter
         self.tol = tol
+        self.verbose = verbose
+
+        self.use_normalized = use_normalized
 
         self.subpopulations = None
         self.subpopulation_sizes = None
@@ -89,28 +109,36 @@ class GAM:
         self.scoring_method = scoring_method
         self.score = None
 
+        if seed:
+            np.random.seed(seed=seed)
+            
     def _read_df_or_list(self):
         """
         Converts attributions to numpy array and feature labels to a list if a pandas dataframe, numpy array, or list is passed in,
 
         Returns
-            attributions (numpy.ndarray): for example, [(.2, .8), (.1, .9)]
-            feature labels: ("height", "weight")
+            attributions (numpy.ndarray or dask.array): for example, [(.2, .8), (.1, .9)]
+            feature labels (list): ("height", "weight")
         """
-        if isinstance(self.attributions, pd.DataFrame):
+        if isinstance(self.attributions, dd.DataFrame):
+            self.feature_labels = self.attributions.columns.tolist()
+            self.attributions = self.attributions.to_dask_array(lengths=True)
+        elif isinstance(self.attributions, pd.DataFrame):
             self.feature_labels = self.attributions.columns.tolist()
             self.attributions = np.asarray(self.attributions.values.tolist())
-        elif isinstance(self.attributions, (np.ndarray, list)) or isinstance(self.feature_labels, (np.ndarray, list)):
+        elif isinstance(self.attributions, (np.ndarray, list)) or isinstance(self.attributions, da.Array) or isinstance(self.feature_labels, (np.ndarray, list)) or isinstance(self.feature_labels, da.Array):
             if (isinstance(self.attributions, (np.ndarray, list)) and self.feature_labels is None) or (self.attributions is None and self.feature_labels is not None):
                 raise ValueError("You must have both 'attributions' and 'feature_labels' if 'attributions' is not a dataframe.")
-            if isinstance(self.attributions, list):
+            elif isinstance(self.attributions, list):
                 self.attributions = np.asarray(self.attributions)
-            elif isinstance(self.attributions, np.ndarray):
+            elif isinstance(self.attributions, (np.ndarray, da.Array)):
                 self.attributions = self.attributions
             if isinstance(self.feature_labels, list):
                 self.feature_labels = self.feature_labels
             elif isinstance(self.feature_labels, np.ndarray):
                 self.feature_labels = self.feature_labels.tolist()
+            elif isinstance(self.feature_labels, da.Array):
+                self.feature_labels = self.feature_labels.compute().tolist()
         else:
             self.attributions = None
             self.feature_labels = None
@@ -156,12 +184,20 @@ class GAM:
                 dist_func=self.distance_function,
                 max_iter=self.max_iter,
                 tol=self.tol,
+                init_medoids=self.init_medoids,
+                swap_medoids=self.swap_medoids,
             )
-            clusters.fit(self.clustering_attributions, verbose=False)
+            clusters.fit(self.clustering_attributions, verbose=self.verbose)
 
             self.subpopulations = clusters.members
             self.subpopulation_sizes = GAM.get_subpopulation_sizes(clusters.members)
             self.explanations = self._get_explanations(clusters.centers)
+            # Making explanations return numerical values instead of dask arrays
+            if isinstance(self.explanations[0][0][1], da.Array):
+                explanations = []
+                for explanation in self.explanations:
+                    explanations.append([(x[0], x[1].compute()) for x in explanation])
+                self.explanations = explanations
         else:
             self.cluster_method(self)
 
@@ -193,7 +229,6 @@ class GAM:
         explanations = []
 
         for center_index in centers:
-            # explanation_weights = self.normalized_attributions[center_index]
             explanation_weights = self.clustering_attributions[center_index]
             explanations.append(list(zip(self.feature_labels, explanation_weights)))
         return explanations
@@ -234,10 +269,9 @@ class GAM:
 
     def generate(self):
         """Clusters local attributions into subpopulations with global explanations"""
-        if self.attributions is not None:
+        if self.attributions_path is None:
             self._read_df_or_list()
         else:
-            # we need to read in attributions from a CSV file, since we don't have any in memory
             self._read_local()
         if self.use_normalized:
             self.clustering_attributions = GAM.normalize(self.attributions)
